module Expr where

import Char
import Scanner
import Parser

data S = CI Integer
       | V String
       | Neg S
       | S :+: S
       | S :-: S
       | S :*: S
       deriving (Eq,Show)

buildSyntaxTree :: String -> S
buildSyntaxTree s = let tokStream = scanner s
                    in head [ x | (x,[]) <- parseE tokStream ]

eval :: S -> Integer
eval (CI i)   = i
eval (V _)    = error "eval (V _) undefined"
eval (Neg a)  = - eval a
eval (a:+:b)  = eval a + eval b
eval (a:-:b)  = eval a - eval b
eval (a:*:b)  = eval a * eval b


parseE, parseT, parseF :: Parse Token S

parseE', parseT' :: S -> Parse Token S

parseE = parseT >-> parseE'

parseE' exp =      succeed exp
              <|>  ((match '+' >*> parseT) `build` (\ (_,e) -> exp :+: e)
                     >-> parseE')               
              <|>  ((match '-' >*> parseT) `build` (\ (_,e) -> exp :-: e)
                     >-> parseE') 

parseT = parseF >-> parseT'

parseT' exp =      succeed exp
              <|>  ((match '*' >*> parseF) `build` (\ (_,e) -> exp :*: e)
                     >-> parseT')

parseF = (number `build` CI) <|> 
         (name   `build` V ) <|>
         (((match '-') >*> parseF) `build` (\ (_,e) -> Neg e )) <|>
         (((match '(') >*> parseE >*> match ')')
                       `build` (\ (_,(e,_)) -> e))
               
