module Parser where

import Char
import Scanner

infixl 4 <|>     -- Alternative von Parsern
infixr 5 >*>     -- Sequenz von Parsern
infixr 5 >->     -- wie a>*>b, b erbt Attr. von a


type Parse a b = [a] -> [(b,[a])]

spot :: (a->Bool) -> Parse a a
spot p (x:xs) | p x = [(x,xs)]
spot _ _ = []

none :: Parse a b
none inp = []

succeed :: b -> Parse a b
succeed val inp = [(val,inp)]

token :: Eq a => a -> Parse a a
token t = spot (==t)

match :: Char -> Parse Token Token
match c = token (Other c)

number :: Parse Token Integer
number (Number i : xs) = [(i,xs)]
number _ = []

name :: Parse Token String
name (Name s : xs) = [(s,xs)]
name _ = []

other :: Parse Token Char
other (Other c : xs) = [(c,xs)]
other _ = []

withLookahead :: Parse a b -> (a->Bool) -> Parse a b
withLookahead p pred s 
 = [ (x,r) | (x,r) <- p s, not (null (spot pred r)) ]

(<|>) :: Parse a b -> Parse a b -> Parse a b
(p1 <|> p2) inp = p1 inp ++ p2 inp

(>*>) :: Parse a b -> Parse a c -> Parse a (b,c)
(p1 >*> p2) inp = [ ((s1,s2),s) | (s1,r) <- p1 inp,
                                  (s2,s) <- p2 r    ]

(>->) :: Parse a b -> (b -> Parse a c) -> Parse a c
(p1 >-> p2) inp = [ (s2,s) | (s1,r) <- p1 inp,
                             (s2,s) <- p2 s1 r    ]

build :: Parse a b -> (b->c) -> Parse a c
build p f inp = [ (f x, rem) | (x,rem) <- p inp ]

list :: Parse a b -> Parse a [b]
list p = (succeed []) <|> ((p >*> list p) `build` (uncurry (:))) 

parseAll :: Eq a => [a] -> Parse a [a]
parseAll set = let p = spot (`elem` set)
               in list p `withLookahead` (`notElem` set)

